<?php
use WebRTB\AddressSplitter\App\Exceptions\SplitAddressException;
use WebRTB\AddressSplitter\App\Split;

add_filter( 'woocommerce_order_actions', function ( $actions ) {
	global $theorder; /** @var $theorder WC_Order */

	foreach ($theorder->get_items() as $item) {
		if ($item instanceof WC_Order_Item_Product && dwd_wc_get_product_data($item->get_product())) {
			$actions['dwd_wc_inject'] = __( 'Send order to Drukwerkdeal', 'dwd-wc' );
			return $actions;
		}
	}
	return $actions;
});

add_action( 'woocommerce_order_action_dwd_wc_inject', function() {
    $url = admin_url("?page=dwd-wc-inject-order&order_id={$_POST['ID']}");
    wp_redirect($url);
    die();
});

function dwd_wc_inject_order(WC_Order $order)
{
	$contents = dwd_wc_get_order_json($order);

	$retVal = DWD_WC_API_Helper::getInstance()->validateOrder(json_decode($contents)->orderLines);
	if(!$retVal) {
		wp_die("Could not inject order, product not valid");
	}

    $dwd_order = DWD_WC_API_Helper::getInstance()->createOrder($contents);
	$order->add_meta_data('dwd_order_request', $contents, true);
	$order->add_meta_data('dwd_order_response', $dwd_order, true);
	$order->add_meta_data( 'dwd_order_uuid', $dwd_order, true);
	$order->add_meta_data('dwd_order_number', '', true);
	$order->save();

	$order->set_status('wc-dwd-creating', "Order send to DWD at " . date("Y-m-d H:i:s"));
	$order->save();
	return $dwd_order;
}

add_action('admin_menu', function() {
	add_dashboard_page(
		__('Inject order', 'dwd-woocommerce'),
		__('Inject order', 'dwd-woocommerce'),
		'manage_options',
		'dwd-wc-inject-order',
		'dwd_wc_inject_order_view'
	);
});

add_action('admin_head', function() {
	remove_submenu_page('index.php', 'dwd-wc-inject-order');
});

add_action('wp_ajax_dwd_wc_get_order_json', function() {
    //wp_send_json_success(['json' => "{\"hello\": \"world\"}"]);
	$order = wc_get_order($_POST['order_id']);
    $json = dwd_wc_get_order_json($order);
    wp_send_json_success(['json' => $json]);
});

add_action('wp_ajax_dwd_wc_inject_order', function() {
    $order = wc_get_order($_POST['order_id']); /** @var $order WC_Order */
    if (!$order instanceof WC_Order) {
        wp_send_json_error(['message' => "Cannot find order {$_POST['order_id']}"]);
        return;
    }

    $json = dwd_wc_inject_order($order);
    wp_send_json_success(['json' => json_encode($json)]);
});

add_action( 'admin_enqueue_scripts', function($hook_suffix) {
    if (stripos($hook_suffix, "dwd-wc-inject-order") !== false) {
	    wp_enqueue_script( 'dwd-wc-admin-inject-order', plugins_url( 'admin/js/inject-order.js', DWD_WOOCOMMERCE_PLUGIN_NAME) );
    }
});

add_action('wp_ajax_dwd_wc_get_order_state', function() {
    $api = DWD_WC_API_Helper::getInstance();
    $order = $api->getOrder($_POST['uuid']);
    if ($order['error']) {
        wp_send_json_success($order);
    }
    wp_send_json_success(['json' => json_encode($order)]);
});

function dwd_wc_get_order_json($order) {
	$dwd_order = [ 'orderLines' => [] ];
	foreach ( $order->get_items() as $item ) {
		if (!$item instanceof WC_Order_Item_Product) {
			continue;
		}
		$product        = $item->get_product();
		$dwd_product_data = dwd_wc_get_product_data($product);
		if (!$dwd_product_data) {
			continue;
		}

		$dwd_order_line = [ 'attributes' => [], 'sku' => $dwd_product_data['uuid'] ];
		$selection = $dwd_product_data['attributes'];

		$selection['quantity'] = $item->get_quantity();
		foreach ( array_keys( $selection ) as $key ) {
			$dwd_order_line['attributes'][] = [
				'attribute' => $key,
				'value'     => $selection[ $key ]
			];
		}

		$file = $item->get_meta('dwd_wc_file', true);
		if ($file) {
			$dwd_order_line['files'] = [['url' => $file]];
		}

		$dwd_order_line = apply_filters('dwd-woocommerce_dwd_order_line', $dwd_order_line, $item, $order);
		if ($dwd_order_line) {
			$dwd_order['orderLines'][] = $dwd_order_line;
		}

	}
	if (count($dwd_order['orderLines']) <= 0) {
		return false;
	}

	$dwd_order['invoiceAddress']  = [
		"firstName"   => get_option('dwd-wc-firstname'),
		"lastName"    => get_option('dwd-wc-lastname'),
		"email"       => get_option('dwd-wc-email'),
		"street"      => get_option('dwd-wc-street'),
		"housenumber" => get_option('dwd-wc-housenumber'),
		"zipcode"     => get_option('dwd-wc-zipcode'),
		"city"        => get_option('dwd-wc-city'),
		"country"     => get_option('dwd-wc-country')
	];

	$housenumber = "";
	try {
		$address = Split::split($order->get_shipping_address_1());
		$street = $address[0];
		$housenumber = $address[1] . $address[2];
	} catch(SplitAddressException $exception) {
		$street = $order->get_shipping_address_1();
	}

	$dwd_order['deliveryAddress'] = [
		"firstName" => $order->get_shipping_first_name(),
		"lastName" => $order->get_shipping_last_name(),
		"email" => get_option("dwd-wc-email"),
		"street" => $street,
		"housenumber" => $housenumber,
		"zipcode" => $order->get_shipping_postcode(),
		"city" => $order->get_shipping_city(),
		"country" => $order->get_shipping_country()
	];
	$dwd_order['deliveryMethod']  = 1;
	$dwd_order['reference']       = "API-ORDER-{$order->get_id()}";
	$dwd_order = apply_filters('dwd-woocommerce_dwd_order', $dwd_order, $order);
	if (!$dwd_order) {
		return false;
	}
	$contents                     = json_encode( $dwd_order );
	return $contents;
}

function dwd_wc_inject_order_view() {
	$order_id = $_GET['order_id'];
	$order = wc_get_order($order_id);
	wp_print_scripts([ 'wp-util' ]);
	?>
	<div class="wrap">
		<h1>Inject order to Drukwerkdeal</h1>
		<p>This function submits the order using your Drukwerkdeal.nl or Printdeal.be account. Payment is done
		on account, so if a file is provided, the order will be processed right away. Cancellation might not
		be possible after submitting.</p>
		<p>After submitting the order, you can leave this page, however it might be useful to wait a few minutes
		the first time you do this to see the result of order creation.</p>
		<p>For debugging information, the API traffic is also displayed below.</p>
		<h2>Order <?php echo $order->get_order_number(); ?></h2>
        <p>The following items will be submitted:</p>
        <ul style="display: block; list-style-type: disc; margin-left: 1em; padding-left: 40px;">
            <?php foreach ($order->get_items() as $item): ?>
                <?php if ($item instanceof WC_Order_Item_Product && dwd_wc_get_product_data($item->get_product())): ?>
                    <li><?php echo $item->get_name(); ?> &bull; <?php echo "{$order->get_currency()} {$item->get_total()}"; ?></li>
                <?php endif; ?>
            <?php endforeach; ?>
        </ul>
        <div>
            <div style="width: 33%; float: left;">

                <h1>Order request <span id="dwd-wc-spinner-req" class="spinner" style="float: none;"></span></h1>
                <textarea readonly style="width: 90%; height: 200px;" id="dwd-wc-order-req"></textarea>
            </div>
            <div style="width: 33%; float: left;">
                <h1>Order response <span id="dwd-wc-spinner-res" class="spinner" style="float: none;"></span></h1>
                <textarea readonly style="width: 90%; height: 200px;"  id="dwd-wc-order-res"></textarea>
            </div>
            <div style="width: 33%; float: left;">
                <h1>Finished order <span id="dwd-wc-spinner-obj" class="spinner" style="float: none;"></span></h1>
                <textarea readonly style="width: 90%; height: 200px;" id="dwd-wc-order-obj"></textarea>
            </div>
        </div>
    </div>
    <div style="clear: both;"></div>
    <div style="text-align: right; margin-top: 5px; padding-right: 20px;">
        <?php edit_post_link("Back to order", "", "", $order_id, 'button edit-post-link'); ?>
        <button class="button-primary" id="dwd-wc-inject-order" style="top: -10px;">Inject order</button>
    </div>
	<?php
}